/*
 * C6Run_main.c
 */

/*
 * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
 */

/* 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/************************************************************************
*  Defines DSP/Bios main entry point that serves to kick off the        *
*  user supplied main in its own task.                                  *
************************************************************************/


/*  ----------------------------------- Headers for User Code       */
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/*  ----------------------------------- DSP/BIOS Headers            */
#include "C6Runcfg.h"
#include <sys.h>
#include <sem.h>
#include <msgq.h>
#include <pool.h>
#include <gbl.h>
#include <clk.h>
#include <log.h>

/*  ----------------------------------- DSP/BIOS LINK Headers       */
#include <dsplink.h>
#include <failure.h>

/*  ----------------------------------- Headers for CIO Code        */
#include "C6Run_control.h"


/************************************************************
* Explicit External Declarations                            *
************************************************************/

/* FILEID is used by SET_FAILURE_REASON macro. */
#define FILEID  FID_APP_C


/************************************************************
* Local Macro Declarations                                  *
************************************************************/


/************************************************************
* Local Typedef Declarations                                *
************************************************************/


/************************************************************
* Local Function Declarations                               *
************************************************************/


/************************************************************
* Local Variable Definitions                                *
************************************************************/


/************************************************************
* Global Variable Definitions                               *
************************************************************/


/************************************************************
* Global Function Definitions                               *
************************************************************/

void _C6Run_main(int argc, char* argv [])
{
  /* Task handler for TSK_create */
  TSK_Handle task;
  TSK_Attrs  attrs = TSK_ATTRS;

  /* Initialize DSP/BIOS LINK. */
  DSPLINK_init();

  /* Start Control task */
  task = TSK_create(C6RUN_CONTROL_task, &attrs, argc, argv);
  if (task == NULL)
  {
    SET_FAILURE_REASON(SYS_EALLOC);
    LOG_error("Create TSKMESSAGE: Failed", 0);
  }
  
}

// Replacement clock function to use in place of RTS lib version
_CODE_ACCESS clock_t clock(void)   
{
  clock_t result;

  // Calculate absolute time in microseconds
  result = (clock_t) ((CLK_getltime() *  CLK_cpuCyclesPerLtime() * 1000) / GBL_getFrequency());
  
  return result;
}

// Replacement time function to use in place of RTS lib version
_CODE_ACCESS time_t time(time_t *timer) 
{
   time_t result;

   result = (time_t)HOSTtime();
   if(timer) *timer = result;
   return(result);
}


/************************************************************
* Local Function Definitions                                *
************************************************************/


/***********************************************************
* End file                                                 *
***********************************************************/
